@@ -13,9 +13,6 @@ use crate::{
     util::{BaudPeriph, RccPeriph},
 };
 
-#[cfg(any(feature = "f3", feature = "l4"))]
-use crate::util::DmaPeriph;
-
 use core::ops::Deref;
 
 use cortex_m::interrupt::free;
@@ -33,7 +30,7 @@ use crate::pac::dma as dma_p;
 use crate::pac::dma1 as dma_p;
 
 #[cfg(not(any(feature = "f4", feature = "l5")))]
-use crate::dma::{self, Dma, DmaChannel};
+use crate::dma::{self, ChannelCfg, Dma, DmaChannel};
 
 #[cfg(any(feature = "f3", feature = "l4"))]
 use crate::dma::DmaInput;
@@ -159,7 +156,6 @@ pub struct Usart<R> {
 
 impl<R> Usart<R>
 where
-    // R: Deref<Target = pac::usart1::RegisterBlock> + DmaPeriph + RccPeriph + BaudPeriph,
     R: Deref<Target = pac::usart1::RegisterBlock> + RccPeriph + BaudPeriph,
 {
     /// Initialize a U[s]ART peripheral, including configuration register writes, and enabling and
@@ -177,9 +173,6 @@ where
         result.regs.cr1.modify(|_, w| w.ue().clear_bit());
         while result.regs.cr1.read().ue().bit_is_set() {}
 
-        #[cfg(not(any(feature = "f3", feature = "f4")))]
-        let word_len_bits = result.config.word_len.bits();
-
         // Set up transmission. See L44 RM, section 38.5.2: "Character Transmission Procedures".
         // 1. Program the M bits in USART_CR1 to define the word length.
 
@@ -372,11 +365,16 @@ where
         }
     }
 
-    #[cfg(not(any(feature = "g0", feature = "h7", feature = "f4", feature = "l5")))]
+    #[cfg(not(any(feature = "g0", feature = "f4", feature = "l5")))]
     /// Transmit data using DMA. (L44 RM, section 38.5.15)
     /// Note that the `channel` argument is only used on F3 and L4.
-    pub unsafe fn write_dma<D>(&mut self, buf: &[u8], channel: DmaChannel, dma: &mut Dma<D>)
-    where
+    pub unsafe fn write_dma<D>(
+        &mut self,
+        buf: &[u8],
+        channel: DmaChannel,
+        channel_cfg: ChannelCfg,
+        dma: &mut Dma<D>,
+    ) where
         D: Deref<Target = dma_p::RegisterBlock>,
     {
         let (ptr, len) = (buf.as_ptr(), buf.len());
@@ -389,7 +387,16 @@ where
         #[cfg(feature = "l4")]
         R::write_sel(dma);
 
-        // todo: Pri and Circular as args?
+        #[cfg(feature = "h7")]
+        let num_data = len as u32;
+        #[cfg(not(feature = "h7"))]
+        let num_data = len as u16;
+
+        // "DMA mode can be enabled for transmission by setting DMAT bit in the USART_CR3
+        // register. Data is loaded from a SRAM area configured using the DMA peripheral (refer to
+        // Section 11: Direct memory access controller (DMA) on page 295) to the USART_TDR
+        // register whenever the TXE bit is set."
+        self.regs.cr3.modify(|_, w| w.dmat().set_bit());
 
         dma.cfg_channel(
             channel,
@@ -402,13 +409,13 @@ where
             // after each TXE event.
             ptr as u32,
             // 3. Configure the total number of bytes to be transferred to the DMA control register.
-            len as u16,
+            num_data,
             dma::Direction::ReadFromMem,
             // 4. Configure the channel priority in the DMA control register
             // (Handled by `ChannelCfg::default())`
             dma::DataSize::S8,
             dma::DataSize::S8,
-            Default::default(),
+            channel_cfg,
         );
 
         // 5. Configure DMA interrupt generation after half/ full transfer as required by the
@@ -430,20 +437,18 @@ where
         // disabling the USART or entering Stop mode. Software must wait until TC=1. The TC flag
         // remains cleared during all data transfers and it is set by hardware at the end of transmission
         // of the last frame.
-
-        // (this RM quote isn't part of the steps and section the others above are)
-        // "DMA mode can be enabled for transmission by setting DMAT bit in the USART_CR3
-        // register. Data is loaded from a SRAM area configured using the DMA peripheral (refer to
-        // Section 11: Direct memory access controller (DMA) on page 295) to the USART_TDR
-        // register whenever the TXE bit is set."
-        self.regs.cr3.modify(|_, w| w.dmat().set_bit());
     }
 
     #[cfg(not(any(feature = "g0", feature = "f4", feature = "l5")))]
-    /// Receive data using DMA. (L44 RM, section 38.5.15)
+    /// Receive data using DMA. (L44 RM, section 38.5.15; G4 RM section 37.5.19.
     /// Note that the `channel` argument is only used on F3 and L4.
-    pub unsafe fn read_dma<D>(&mut self, buf: &mut [u8], channel: DmaChannel, dma: &mut Dma<D>)
-    where
+    pub unsafe fn read_dma<D>(
+        &mut self,
+        buf: &mut [u8],
+        channel: DmaChannel,
+        channel_cfg: ChannelCfg,
+        dma: &mut Dma<D>,
+    ) where
         D: Deref<Target = dma_p::RegisterBlock>,
     {
         let (ptr, len) = (buf.as_mut_ptr(), buf.len());
@@ -458,12 +463,15 @@ where
         #[cfg(not(feature = "h7"))]
         let num_data = len as u16;
 
+        // DMA mode can be enabled for reception by setting the DMAR bit in USART_CR3 register.
+        self.regs.cr3.modify(|_, w| w.dmar().set_bit());
+
         dma.cfg_channel(
             channel,
             // 1. Write the USART_RDR register address in the DMA control register to configure it as
             // the source of the transfer. The data is moved from this address to the memory after
             // each RXNE event.
-            &self.regs.tdr as *const _ as u32,
+            &self.regs.rdr as *const _ as u32,
             // 2. Write the memory address in the DMA control register to configure it as the destination
             // of the transfer. The data is loaded from USART_RDR to this memory area after each
             // RXNE event.
@@ -473,16 +481,16 @@ where
             dma::Direction::ReadFromPeriph,
             dma::DataSize::S8,
             dma::DataSize::S8,
-            Default::default(),
+            channel_cfg,
         );
 
-        self.regs.cr3.modify(|_, w| w.dmar().set_bit());
+        // 4. Configure the channel priority in the DMA control register
+        // (Handled in cfg)
 
         // 5. Configure interrupt generation after half/ full transfer as required by the application.
-        // todo (Let the user call `dma.setup_interrupt()`? But that requires they know
-        // todo which channel of DMA usart tx is on.
+        // (Handled by user code))
 
-        // 6. Activate the channel in the DMA control register.
+        // 6. Activate the channel in the DMA control register. (Handled by `cfg_channel` above).
         // When the number of data transfers programmed in the DMA Controller is reached, the DMA
         // controller generates an interrupt on the DMA channel interrupt vector.
         // (Handled in above fn call)
@@ -502,12 +510,12 @@ where
     #[cfg(not(feature = "f4"))]
     /// Enable a specific type of interrupt.
     pub fn enable_interrupt(&mut self, interrupt: UsartInterrupt) {
-        // Disable the UART to allow writing the `add` and `addm7` bits
-        self.regs.cr1.modify(|_, w| w.ue().clear_bit());
-        while self.regs.cr1.read().ue().bit_is_set() {}
-
         match interrupt {
             UsartInterrupt::CharDetect(char) => {
+                // Disable the UART to allow writing the `add` and `addm7` bits
+                self.regs.cr1.modify(|_, w| w.ue().clear_bit());
+                while self.regs.cr1.read().ue().bit_is_set() {}
+
                 // Enable character-detecting UART interrupt
                 self.regs.cr1.modify(|_, w| w.cmie().set_bit());
 
@@ -518,12 +526,16 @@ where
                     cfg_if! {
                         if #[cfg(any(feature = "f3", feature = "l4", feature = "h7", feature = "wl"))] {
                             w.add().bits(char)
-                        } else { // todo: Is this right, or backwards?
-                            w.add0_3().bits(char & 0b1111);
-                            w.add4_7().bits(char & (0b1111 << 4))
+                        // } else if #[cfg(feature = "g4")] {
+                        //     w.add0_3().bits(char) // PAC error. Should be just like above. (?)
+                        } else { // note: G4 should be like above, but use below due to PAC error. (Should be equiv)
+                            w.add0_3().bits(char);
+                            w.add4_7().bits(char >> 4)
                         }
                     }
                 });
+
+                self.regs.cr1.modify(|_, w| w.ue().set_bit());
             }
             UsartInterrupt::Cts => {
                 self.regs.cr3.modify(|_, w| w.ctsie().set_bit());
@@ -555,6 +567,7 @@ where
             #[cfg(not(any(feature = "f3", feature = "l4")))]
             UsartInterrupt::Tcbgt => {
                 self.regs.cr3.modify(|_, w| w.tcbgtie().set_bit());
+                self.regs.cr3.modify(|_, w| w.tcbgtie().set_bit());
             }
             UsartInterrupt::TransmissionComplete => {
                 self.regs.cr1.modify(|_, w| w.tcie().set_bit());
