@@ -6,12 +6,15 @@
 
 // Based on `stm3h7xx-hal`
 
-use crate::pac::{self, PWR, RCC};
+use crate::{
+    clocks::Clocks,
+    gpio::Pin,
+    pac::{self, PWR, RCC}
+};
 
 pub use synopsys_usb_otg::UsbBus;
 use synopsys_usb_otg::UsbPeripheral;
 
-// #[cfg(not(feature = "h7455"))]
 pub struct USB1 {
     pub usb_global: pac::OTG1_HS_GLOBAL,
     pub usb_device: pac::OTG1_HS_DEVICE,
@@ -22,7 +25,6 @@ pub struct USB1 {
     pub hclk: u32,
 }
 
-// #[cfg(not(feature = "rm0455"))]
 pub struct USB2 {
     pub usb_global: pac::OTG2_HS_GLOBAL,
     pub usb_device: pac::OTG2_HS_DEVICE,
@@ -38,7 +40,7 @@ macro_rules! usb_peripheral {
         unsafe impl Sync for $USB {}
 
         unsafe impl UsbPeripheral for $USB {
-            const REGISTERS: *const () = stm32::$GLOBAL::ptr() as *const ();
+            const REGISTERS: *const () = pac::$GLOBAL::ptr() as *const ();
 
             const HIGH_SPEED: bool = true;
             const FIFO_DEPTH_WORDS: usize = 1024;
@@ -66,7 +68,7 @@ macro_rules! usb_peripheral {
                 // than 30MHz. See RM0433 Rev 7. Section 57.4.4. This is checked
                 // by the UsbBus implementation in synopsys-usb-otg.
 
-                self.hclk.0
+                self.hclk
             }
         }
     };
@@ -85,70 +87,36 @@ usb_peripheral! {
 pub type Usb2BusType = UsbBus<USB2>;
 
 pub struct USB1_ULPI {
-    pub usb_global: stm32::OTG1_HS_GLOBAL,
-    pub usb_device: stm32::OTG1_HS_DEVICE,
-    pub usb_pwrclk: stm32::OTG1_HS_PWRCLK,
-    pub prec: rcc::rec::Usb1Otg,
-    pub hclk: Hertz,
-    pub ulpi_clk: PA5<Alternate<AF10>>,
-    pub ulpi_dir: Usb1UlpiDirPin,
-    pub ulpi_nxt: Usb1UlpiNxtPin,
-    pub ulpi_stp: PC0<Alternate<AF10>>,
-    pub ulpi_d0: PA3<Alternate<AF10>>,
-    pub ulpi_d1: PB0<Alternate<AF10>>,
-    pub ulpi_d2: PB1<Alternate<AF10>>,
-    pub ulpi_d3: PB10<Alternate<AF10>>,
-    pub ulpi_d4: PB11<Alternate<AF10>>,
-    pub ulpi_d5: PB12<Alternate<AF10>>,
-    pub ulpi_d6: PB13<Alternate<AF10>>,
-    pub ulpi_d7: PB5<Alternate<AF10>>,
-}
-
-pub enum Usb1UlpiDirPin {
-    PC2(PC2<Alternate<AF10>>),
-    PI11(PI11<Alternate<AF10>>),
-}
-
-impl From<PI11<Alternate<AF10>>> for Usb1UlpiDirPin {
-    fn from(v: PI11<Alternate<AF10>>) -> Self {
-        Usb1UlpiDirPin::PI11(v)
-    }
-}
-
-impl From<PC2<Alternate<AF10>>> for Usb1UlpiDirPin {
-    fn from(v: PC2<Alternate<AF10>>) -> Self {
-        Usb1UlpiDirPin::PC2(v)
-    }
-}
-
-pub enum Usb1UlpiNxtPin {
-    PC3(PC3<Alternate<AF10>>),
-    PH4(PH4<Alternate<AF10>>),
-}
-
-impl From<PH4<Alternate<AF10>>> for Usb1UlpiNxtPin {
-    fn from(v: PH4<Alternate<AF10>>) -> Self {
-        Usb1UlpiNxtPin::PH4(v)
-    }
-}
-
-impl From<PC3<Alternate<AF10>>> for Usb1UlpiNxtPin {
-    fn from(v: PC3<Alternate<AF10>>) -> Self {
-        Usb1UlpiNxtPin::PC3(v)
-    }
+    pub usb_global: pac::OTG1_HS_GLOBAL,
+    pub usb_device: pac::OTG1_HS_DEVICE,
+    pub usb_pwrclk: pac::OTG1_HS_PWRCLK,
+    pub prec: u32, // todo: What should this be? Maybe d2ccip2 / cdccip2 ?
+    pub hclk: u32,
+    pub ulpi_clk: Pin,
+    pub ulpi_dir: Pin,
+    pub ulpi_nxt: Pin,
+    pub ulpi_stp: Pin,
+    pub ulpi_d0: Pin,
+    pub ulpi_d1: Pin,
+    pub ulpi_d2: Pin,
+    pub ulpi_d3: Pin,
+    pub ulpi_d4: Pin,
+    pub ulpi_d5: Pin,
+    pub ulpi_d6: Pin,
+    pub ulpi_d7: Pin,
 }
 
 unsafe impl Sync for USB1_ULPI {}
 
 unsafe impl UsbPeripheral for USB1_ULPI {
-    const REGISTERS: *const () = stm32::OTG1_HS_GLOBAL::ptr() as *const ();
+    const REGISTERS: *const () = pac::OTG1_HS_GLOBAL::ptr() as *const ();
 
     const HIGH_SPEED: bool = true;
     const FIFO_DEPTH_WORDS: usize = 1024;
     const ENDPOINT_COUNT: usize = 9;
 
     fn enable() {
-        let rcc = unsafe { &*stm32::RCC::ptr() };
+        let rcc = unsafe { &*pac::RCC::ptr() };
 
         cortex_m::interrupt::free(|_| {
             // Enable USB peripheral
@@ -164,7 +132,7 @@ unsafe impl UsbPeripheral for USB1_ULPI {
     }
 
     fn ahb_frequency_hz(&self) -> u32 {
-        self.hclk.0
+        self.hclk
     }
 
     fn phy_type(&self) -> synopsys_usb_otg::PhyType {
